/*******************************************************************************
*
*	TITLE:		defense.c 
*
*	VERSION:	0.1                           
*
*	DATE:		02-12-2007
*
*	AUTHOR:		Team 1511
*
*	DESCRIPTION:	This file contains functions to drive the various autonomous modes of the robot.
********************************************************************************
*
*	CHANGE LOG:
*
*	DATE         REV  DESCRIPTION
*	-----------  ---  ----------------------------------------------------------
* 	12-Feb-2007  0.1  Initial revision.
*
*******************************************************************************/

#include <stdio.h>
#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"
#include "user_routines.h"
#include "serial_ports.h"
#include "encoder.h"
#include "motion.h"
#include "defense.h"
#include "camera.h"
#include "tracking.h"

/* Distance to cross width (short dimension) of field */
#define TICKS_CROSS_WIDTH      (22L * TICKS_PER_FOOT)
/*
 * Distance to go from start (near wall) to approx 
 * center field
 */
#define TICKS_HALF_FIELD    (25L * TICKS_PER_FOOT)



typedef enum
{
  TURN_LEFT,
  TURN_RIGHT,
  TURN_NONE
} Turn;

typedef struct
{
  const rom long ticks;
  const rom Turn turn;
} Movement;  

typedef struct
{
  const rom Movement* steps;
  unsigned char length;
} Strategy;


// MODE 01
static const rom Movement Block_Rack_Left_Side_Left[] =   // Start on left side of field and block left side of rack.
{
  { -TICKS_HALF_FIELD, TURN_NONE },            // Drive up to rack center line and stay there.
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT }    // Turn sideways for stability
};

// MODE 02
static const rom Movement Block_Rack_Left_Side_Right[] =   // Start on right side of field and block left side of rack.
{
  { -TICKS_HALF_FIELD / 2, TURN_NONE },        // Drive up a bit
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT },    // Turn
  { -TICKS_CROSS_WIDTH, TURN_NONE },           // Get to left side of field
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT },   // Turn
  { -TICKS_HALF_FIELD / 2, TURN_NONE }         // Block left side of rack
};

// MODE 03
static const rom Movement Block_Rack_Left_Side_Middle[] =   // Start in middle of field and block left side of rack.
{
  { -TICKS_HALF_FIELD / 2, TURN_NONE },         // Drive up a bit
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT },    // Turn
  { -TICKS_CROSS_WIDTH / 2, TURN_NONE },        // Get to left side of field
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT },   // Turn
  { -TICKS_HALF_FIELD / 2, TURN_NONE }          // Block left side of rack
};

// MODE 04
static const rom Movement Block_Rack_Right_Side_Left[] =   // Start on left side of field and block right side of rack.
{
  { -TICKS_HALF_FIELD / 2, TURN_NONE },         // Drive up a bit
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT },   // Turn
  { -TICKS_CROSS_WIDTH, TURN_NONE },        // Get to right side of field
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT },    // Turn
  { -TICKS_HALF_FIELD / 2, TURN_NONE }          // Block right side of rack
};

// MODE 05
static const rom Movement Block_Rack_Right_Side_Right[] =   // Start on right side of field and block right side of rack.
{
  { -TICKS_HALF_FIELD, TURN_NONE },            // Drive up to rack center line and stay there.
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT }    // Turn sideways for stability
};

// MODE 06
static const rom Movement Block_Rack_Right_Side_Middle[] =   // Start in middle of field and block right side of rack.
{
  { -TICKS_HALF_FIELD / 2, TURN_NONE },        // Drive up a bit
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT },   // Turn
  { -TICKS_CROSS_WIDTH / 2, TURN_NONE },       // Get to right side of field
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT },    // Turn
  { -TICKS_HALF_FIELD / 2, TURN_NONE }          // Block right side of rack
};

// MODE 07
static const rom Movement Block_Rack_Near_Side_Left[] =    // Start on left side of field and block near side of rack.
{
  { -TICKS_HALF_FIELD / 2, TURN_NONE },        // Drive up a bit
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT },   // Turn
  { -TICKS_CROSS_WIDTH / 2, TURN_NONE },       // Drive to middle of field
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT },    // Turn
  { -5 * TICKS_PER_FOOT, TURN_NONE },          // Drive up to front of rack
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT }     // Block with wide side of robot
};

// MODE 10
static const rom Movement Block_Rack_Near_Side_Right[] =    // Start on right side of field and block near side of rack.
{
  { -TICKS_HALF_FIELD / 2, TURN_NONE },        // Drive up a bit
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT },    // Turn
  { -TICKS_CROSS_WIDTH / 2, TURN_NONE },       // Drive to middle of field
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT },   // Turn
  { -5 * TICKS_PER_FOOT, TURN_NONE },          // Drive up to front of rack
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT }     // Block with wide side of robot
};

// MODE 11
static const rom Movement Block_Rack_Near_Side_Middle[] =    // Start in middle of field and block near side of rack.
{
  { -15L * TICKS_PER_FOOT, TURN_NONE },        // Drive up to front of rack
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT }     // Block with wide side of robot
};

// MODE 12
static const rom Movement Block_Rack_Far_Side_Left[] =     // Start on left side of field and block far side of rack.
{
  { -(TICKS_HALF_FIELD + 4 * TICKS_PER_FOOT), TURN_NONE },     // Drive down side of field
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT },                 // Turn
  { -TICKS_CROSS_WIDTH / 2, TURN_NONE },                      // Block middle of rack with long side of robot
};

// MODE 13
static const rom Movement Block_Rack_Far_Side_Right[] =     // Start on right side of field and block far side of rack.
{
  { -(TICKS_HALF_FIELD + 4 * TICKS_PER_FOOT), TURN_NONE },      // Drive down side of field
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT },                   // Turn
  { -TICKS_CROSS_WIDTH / 2, TURN_NONE },                      // Block middle of rack with long side of robot
};

// MODE 14
static const rom Movement Block_Rack_Far_Side_Middle[] =     // Start on left side of field and block far side of rack.
{
  { -TICKS_HALF_FIELD / 2, TURN_NONE },                          // Drive to middle of alliance side
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT },                      // Turn
  { -TICKS_CROSS_WIDTH / 2, TURN_NONE },                         // Drive to side of field
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT },                     // Turn to face opposing side
  { -(TICKS_HALF_FIELD / 2 + 4 * TICKS_PER_FOOT), TURN_NONE },     // Drive to other side of field
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT },                     // Turn
  { -TICKS_CROSS_WIDTH / 2, TURN_NONE },                         // Block middle of rack wiht long side of robot
};

// MODE 15
static const rom Movement Shake_Rack_Left_Side_Left[] =    // Start on left side of field and shake left side of rack multiple times. 
{
  { -TICKS_HALF_FIELD, TURN_NONE },            // Drive up to side of rack
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT },   // Face rack side
  { -8 * TICKS_PER_FOOT, TURN_NONE },          // Hit once
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up a bit
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit again
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up again
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit once more
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up to give drivers space
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT }    // Align nicely for drivers
};

// MODE 16
static const rom Movement Shake_Rack_Left_Side_Right[] =    // Start on right side of field and shake left side of rack multiple times. 
{
  { -TICKS_HALF_FIELD / 2, TURN_NONE },        // Drive up a bit
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT },    // Turn perpendicular
  { -TICKS_CROSS_WIDTH / 2, TURN_NONE },       // Drive to left side
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT },   // Turn again
  { -TICKS_HALF_FIELD / 2, TURN_NONE },        // Drive up to side of rack
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT },   // Face rack side
  { -8 * TICKS_PER_FOOT, TURN_NONE },          // Hit once
  { -3 * TICKS_PER_FOOT, TURN_NONE },         // Back up a bit
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit again
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up again
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit once more
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up to give drivers space
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT }    // Align nicely for drivers
};

// MODE 17
static const rom Movement Shake_Rack_Left_Side_Middle[] =    // Start in middle of field and shake left side of rack multiple times. 
{
  { -TICKS_HALF_FIELD / 2, TURN_NONE },        // Drive up a bit
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT },    // Turn perpendicular
  { -TICKS_CROSS_WIDTH / 2, TURN_NONE },       // Drive to left side
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT },   // Turn again
  { -TICKS_HALF_FIELD / 2, TURN_NONE },        // Drive up to side of rack
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT },   // Face rack side
  { -8 * TICKS_PER_FOOT, TURN_NONE },          // Hit once
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up a bit
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit again
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up again
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit once more
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up to give drivers space
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT }    // Align nicely for drivers
};

// MODE 20
static const rom Movement Shake_Rack_Right_Side_Left[] =    // Start on left side of field and shake right side of rack multiple times. 
{
  { -TICKS_HALF_FIELD / 2, TURN_NONE },        // Drive up a bit
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT },   // Turn perpendicular
  { -TICKS_CROSS_WIDTH / 2, TURN_NONE },       // Drive to left side
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT },    // Turn again
  { -TICKS_HALF_FIELD / 2, TURN_NONE },        // Drive up to side of rack
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT },    // Face rack side
  { -8 * TICKS_PER_FOOT, TURN_NONE },          // Hit once
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up a bit
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit again
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up again
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit once more
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up to give drivers space
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT }     // Align nicely for drivers
};

// MODE 21
static const rom Movement Shake_Rack_Right_Side_Right[] =    // Start on left side of field and shake left side of rack multiple times. 
{
  { -TICKS_HALF_FIELD, TURN_NONE },            // Drive up to side of rack
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT },    // Face rack side
  { -8 * TICKS_PER_FOOT, TURN_NONE },          // Hit once
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up a bit
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit again
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up again
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit once more
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up to give drivers space
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT }     // Align nicely for drivers
};

// MODE 22
static const rom Movement Shake_Rack_Right_Side_Middle[] =    // Start in middle of field and shake right side of rack multiple times. 
{
  { -TICKS_HALF_FIELD / 2, TURN_NONE },        // Drive up a bit
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT },   // Turn perpendicular
  { -TICKS_CROSS_WIDTH / 2, TURN_NONE },       // Drive to left side
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT },    // Turn again
  { -TICKS_HALF_FIELD / 2, TURN_NONE },        // Drive up to side of rack
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT },    // Face rack side
  { -8 * TICKS_PER_FOOT, TURN_NONE },          // Hit once
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up a bit
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit again
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up again
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit once more
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up to give drivers space
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT }     // Align nicely for drivers
};

// MODE 23
static const rom Movement Shake_Rack_Near_Side_Left[] =    // Start on left side of field and shake near side of rack multiple times.
{
  { -TICKS_HALF_FIELD / 2, TURN_NONE },        // Go forward a bit for turning room
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT },   // Turn perpendicular to field
  { -TICKS_CROSS_WIDTH / 2, TURN_NONE },       // Drive to position perpendicular to rack center
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT},     // Turn back to face rack
  { -8 * TICKS_PER_FOOT, TURN_NONE },          // Drive up and hit rack once
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up a bit
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit rack again
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up again
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit again
  { 3 * TICKS_PER_FOOT, TURN_NONE }          // Back up once more to give drivers some room to start teleoperated mode
};

// MODE 24
static const rom Movement Shake_Rack_Near_Side_Right[] =    // Start on right side of field and shake near side of rack multiple times.
{
  { -TICKS_HALF_FIELD / 2, TURN_NONE },        // Go forward a bit for turning room
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT },    // Turn perpendicular to field
  { -TICKS_CROSS_WIDTH / 2, TURN_NONE },       // Drive to position perpendicular to rack center
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT},    // Turn back to face rack
  { -8 * TICKS_PER_FOOT, TURN_NONE },          // Drive up and hit rack once
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up a bit
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit rack again
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up again
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit again
  { 3 * TICKS_PER_FOOT, TURN_NONE }          // Back up once more to give drivers some room to start teleoperated mode
};

// MODE 25
static const rom Movement Shake_Rack_Near_Side_Middle[] =    // Start in middle of field and shake near side of rack multiple times.
{
  { -17 * TICKS_PER_FOOT, TURN_NONE },         // Drive up and hit rack once
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up a bit
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit rack again
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up again
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit again
  { 3 * TICKS_PER_FOOT, TURN_NONE }          // Back up once more to give drivers some room to start teleoperated mode
};

// MODE 26
static const rom Movement Shake_Rack_Far_Side_Left[] =   // Start on left side of field and shake far side of rack multiple times.
{
  { -(TICKS_HALF_FIELD + 4 * TICKS_PER_FOOT), TURN_NONE },
                                              // Drive down side of field
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT },   // Turn
  { -TICKS_CROSS_WIDTH / 2, TURN_NONE },       // Get to far side of rack
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT },   // Turn towards rack
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit rack
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit rack again
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up again
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit rack again
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up once more to give drivers some room to start teleoperated mode
};

// MODE 27
static const rom Movement Shake_Rack_Far_Side_Right[] =   // Start on right side of field and shake far side of rack multiple times.
{
  { -(TICKS_HALF_FIELD + 4 * TICKS_PER_FOOT), TURN_NONE },
                                              // Drive down side of field
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT },    // Turn
  { -TICKS_CROSS_WIDTH / 2, TURN_NONE },       // Get to far side of rack
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT },    // Turn towards rack
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit rack
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit rack again
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up again
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit rack again
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up once more to give drivers some room to start teleoperated mode
};

// MODE 30
static const rom Movement Shake_Rack_Far_Side_Middle[] =   // Start in middle side of field and shake far side of rack multiple times.
{
  { -TICKS_HALF_FIELD / 2, TURN_NONE },        // Drive down a bit
  { -TICKS_PER_NINETY_DEGREES, TURN_LEFT },    // Turn
  { -TICKS_CROSS_WIDTH / 2, TURN_NONE },       // Drive to left side of field
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT },   // Turn to face opposing side
  { -(TICKS_HALF_FIELD / 2 + 4 * TICKS_PER_FOOT), TURN_NONE },
                                              // Drive to other side of field
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT },   // Turn
  { -TICKS_CROSS_WIDTH / 2, TURN_NONE },       // Get to far side of rack
  { -TICKS_PER_NINETY_DEGREES, TURN_RIGHT },   // Turn towards rack
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit rack
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit rack again
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up again
  { -4 * TICKS_PER_FOOT, TURN_NONE },          // Hit rack again
  { 3 * TICKS_PER_FOOT, TURN_NONE },         // Back up once more to give drivers some room to start teleoperated mode
};

// MODE 31
static const rom Movement Multi_Block_Rack_Far_Side_Left[] =  // Start on left side of field and block far side of rack, moving back and forth.
{
  { -(TICKS_HALF_FIELD + 4 * TICKS_PER_FOOT), TURN_NONE },     // Drive down side of field
  { TICKS_PER_NINETY_DEGREES, TURN_RIGHT },                 // Turn
  { -(TICKS_CROSS_WIDTH / 2 + 4 * TICKS_PER_FOOT), TURN_NONE },
                                                            // Initial block attack
  { -TICKS_PER_FOOT * -8, TURN_NONE },                       // Second block
  { -TICKS_PER_FOOT * 8, TURN_NONE },                        // Third
  { -TICKS_PER_FOOT * -8, TURN_NONE },                       // Fourth
  { -TICKS_PER_FOOT * 8, TURN_NONE },                        // Fifth
  { -TICKS_PER_FOOT * -8, TURN_NONE },                       // Sixth
  { -TICKS_PER_FOOT * 8, TURN_NONE },                        // Seventh
  { -TICKS_PER_FOOT * -4, TURN_NONE }                        // Return to center and maintain block
};

// MODE 32
static const rom Movement Multi_Block_Rack_Far_Side_Right[] =  // Start on right side of field and block far side of rack, moving back and forth.
{
  { -(TICKS_HALF_FIELD + 4 * TICKS_PER_FOOT), TURN_NONE },     // Drive down side of field
  { TICKS_PER_NINETY_DEGREES, TURN_LEFT },                  // Turn
  { -(TICKS_CROSS_WIDTH / 2 + 4 * TICKS_PER_FOOT), TURN_NONE },
                                                            // Initial block
  { -TICKS_PER_FOOT * -8, TURN_NONE },                       // Second block
  { -TICKS_PER_FOOT * 8, TURN_NONE },                        // Third
  { -TICKS_PER_FOOT * -8, TURN_NONE },                       // Fourth
  { -TICKS_PER_FOOT * 8, TURN_NONE },                        // Fifth
  { -TICKS_PER_FOOT * -8, TURN_NONE },                       // Sixth
  { -TICKS_PER_FOOT * 8, TURN_NONE },                        // Seventh
  { -TICKS_PER_FOOT * -4, TURN_NONE }                        // Return to center and maintain block                                                             // Block middle of rack with long side of robot
};

// MODE 33
static const rom Movement Knock_Down_Tubes_Left[] =        // Start with alliance wall to left and knock down tubes.
// NOTE: For this mode, the robot should be aligned parallel and centered with the first tube along the wall.
//       The distance from the wall to the robot should be slightly more than the length of the robot, so that it can spin 
//       freely but will still knock down the tubes when spinning.
{
  { TICKS_PER_NINETY_DEGREES * 4, TURN_LEFT },              // Spin and knock down first tube.
  { 32 * TICKS_PER_INCH, TURN_NONE },                       // Move forward to next tube.
  { TICKS_PER_NINETY_DEGREES * 4, TURN_LEFT },              // Spin and knock down next tube.
  { 32 * TICKS_PER_INCH, TURN_NONE },                       // Move forward to next tube.
  { TICKS_PER_NINETY_DEGREES * 4, TURN_LEFT },              // Lather, rinse, repeat.
  { 32 * TICKS_PER_INCH, TURN_NONE },
  { TICKS_PER_NINETY_DEGREES * 4, TURN_LEFT },
  { 32 * TICKS_PER_INCH, TURN_NONE },
  { TICKS_PER_NINETY_DEGREES * 4, TURN_LEFT },
  { 32 * TICKS_PER_INCH, TURN_NONE },
  { TICKS_PER_NINETY_DEGREES * 4, TURN_LEFT },
  { 32 * TICKS_PER_INCH, TURN_NONE },
  { TICKS_PER_NINETY_DEGREES * 4, TURN_LEFT },
  { 32 * TICKS_PER_INCH, TURN_NONE },
  { TICKS_PER_NINETY_DEGREES * 4, TURN_LEFT },
  { 32 * TICKS_PER_INCH, TURN_NONE },
  { TICKS_PER_NINETY_DEGREES * 4, TURN_LEFT }
};

// MODE 34
static const rom Movement Knock_Down_Tubes_Right[] =       // Start with alliance wall to right and knock down tubes.
// NOTE: For this mode, the robot should be aligned parallel and centered with the first tube along the wall.
//       The distance from the wall to the robot should be slightly more than the length of the robot, so that it can spin 
//       freely but will still knock down the tubes when spinning.
{
  { TICKS_PER_NINETY_DEGREES * 4, TURN_RIGHT },             // Spin and knock down first tube.
  { 32 * TICKS_PER_INCH, TURN_NONE },                       // Move forward to next tube.
  { TICKS_PER_NINETY_DEGREES * 4, TURN_RIGHT },             // Spin and knock down next tube.
  { 32 * TICKS_PER_INCH, TURN_NONE },                       // Move forward to next tube.
  { TICKS_PER_NINETY_DEGREES * 4, TURN_RIGHT },             // Lather, rinse, repeat.
  { 32 * TICKS_PER_INCH, TURN_NONE },
  { TICKS_PER_NINETY_DEGREES * 4, TURN_RIGHT },
  { 32 * TICKS_PER_INCH, TURN_NONE },
  { TICKS_PER_NINETY_DEGREES * 4, TURN_RIGHT },
  { 32 * TICKS_PER_INCH, TURN_NONE },
  { TICKS_PER_NINETY_DEGREES * 4, TURN_RIGHT },
  { 32 * TICKS_PER_INCH, TURN_NONE },
  { TICKS_PER_NINETY_DEGREES * 4, TURN_RIGHT },
  { 32 * TICKS_PER_INCH, TURN_NONE },
  { TICKS_PER_NINETY_DEGREES * 4, TURN_RIGHT },
  { 32 * TICKS_PER_INCH, TURN_NONE },
  { TICKS_PER_NINETY_DEGREES * 4, TURN_RIGHT }
};

// MODE 35
static const rom Movement Turn_Right[] = {
  { TICKS_PER_NINETY_DEGREES, TURN_RIGHT }
};



static unsigned char Defensive_Autonomous(Strategy* strategy);


unsigned char Run_Defensive_Autonomous(int mode)
{
  Strategy strategy;
  
  switch (mode)
  {
    case 1:
      strategy.steps = Block_Rack_Left_Side_Left;
      strategy.length = sizeof(Block_Rack_Left_Side_Left);
      break;
    case 2:
      strategy.steps = Block_Rack_Left_Side_Right;
      strategy.length = sizeof(Block_Rack_Left_Side_Right);
      break;
    case 3:
      strategy.steps = Block_Rack_Left_Side_Middle;
      strategy.length = sizeof(Block_Rack_Left_Side_Middle);
      break;
    case 4:
      strategy.steps = Block_Rack_Right_Side_Left;
      strategy.length = sizeof(Block_Rack_Right_Side_Left);
      break;
    case 5:
      strategy.steps = Block_Rack_Right_Side_Right;
      strategy.length = sizeof(Block_Rack_Right_Side_Right);
      break;
    case 6:
      strategy.steps = Block_Rack_Right_Side_Middle;
      strategy.length = sizeof(Block_Rack_Right_Side_Middle);
      break;
    case 7:
      strategy.steps = Block_Rack_Near_Side_Left;
      strategy.length = sizeof(Block_Rack_Near_Side_Left);
      break;
    case 10:
      strategy.steps = Block_Rack_Near_Side_Right;
      strategy.length = sizeof(Block_Rack_Near_Side_Right);
      break;
    case 11:
      strategy.steps = Block_Rack_Near_Side_Middle;
      strategy.length = sizeof(Block_Rack_Near_Side_Middle);
      break;
    case 12:
      strategy.steps = Block_Rack_Far_Side_Left;
      strategy.length = sizeof(Block_Rack_Far_Side_Left);
      break;
    case 13:
      strategy.steps = Block_Rack_Far_Side_Right;
      strategy.length = sizeof(Block_Rack_Far_Side_Right);
      break;
    case 14:
      strategy.steps = Block_Rack_Far_Side_Middle;
      strategy.length = sizeof(Block_Rack_Far_Side_Middle);
      break;
    case 15:
      strategy.steps = Shake_Rack_Left_Side_Left;
      strategy.length = sizeof(Shake_Rack_Left_Side_Left);
      break;
    case 16:
      strategy.steps = Shake_Rack_Left_Side_Right;
      strategy.length = sizeof(Shake_Rack_Left_Side_Right);
      break;
    case 17:
      strategy.steps = Shake_Rack_Left_Side_Middle;
      strategy.length = sizeof(Shake_Rack_Left_Side_Middle);
      break;
    case 20:
      strategy.steps = Shake_Rack_Right_Side_Left;
      strategy.length = sizeof(Shake_Rack_Right_Side_Left);
      break;
    case 21:
      strategy.steps = Shake_Rack_Right_Side_Right;
      strategy.length = sizeof(Shake_Rack_Right_Side_Right);
      break;
    case 22:
      strategy.steps = Shake_Rack_Right_Side_Middle;
      strategy.length = sizeof(Shake_Rack_Right_Side_Middle);
      break;
    case 23:
      strategy.steps = Shake_Rack_Near_Side_Left;
      strategy.length = sizeof(Shake_Rack_Near_Side_Left);
      break;
    case 24:
      strategy.steps = Shake_Rack_Near_Side_Right;
      strategy.length = sizeof(Shake_Rack_Near_Side_Right);
      break;
    case 25:
      strategy.steps = Shake_Rack_Near_Side_Middle;
      strategy.length = sizeof(Shake_Rack_Near_Side_Middle);
      break;
    case 26:
      strategy.steps = Shake_Rack_Far_Side_Left;
      strategy.length = sizeof(Shake_Rack_Far_Side_Left);
      break;
    case 27:
      strategy.steps = Shake_Rack_Far_Side_Right;
      strategy.length = sizeof(Shake_Rack_Far_Side_Right);
      break;
    case 30:
      strategy.steps = Shake_Rack_Far_Side_Middle;
      strategy.length = sizeof(Shake_Rack_Far_Side_Middle);
      break;
    case 31:
      strategy.steps = Multi_Block_Rack_Far_Side_Left;
      strategy.length = sizeof(Multi_Block_Rack_Far_Side_Left);
      break;
    case 32:
      strategy.steps = Multi_Block_Rack_Far_Side_Right;
      strategy.length = sizeof(Multi_Block_Rack_Far_Side_Right);
      break;
    case 33:
      strategy.steps = Knock_Down_Tubes_Left;
      strategy.length = sizeof(Knock_Down_Tubes_Left);
      break;
    case 34:
      strategy.steps = Knock_Down_Tubes_Right;
      strategy.length = sizeof(Knock_Down_Tubes_Right);
      break;
    case 35:
      strategy.steps = Turn_Right;
      strategy.length = sizeof(Turn_Right);
      break;
    case 0:   // Do nothing
    default:
      strategy.length = 0;
      break;
  }
  
  strategy.length /= sizeof(Movement);
  printf("Strat len %d\r\n", strategy.length);
  return Defensive_Autonomous(&strategy);
}

static void Defensive_Drive_Init(const rom Movement *mvmt)
{
  switch (mvmt->turn) {
    case TURN_NONE:
      printf("Straight %ld\r\n", mvmt->ticks);
      Init_Drive_Straight(mvmt->ticks);
      break;
    case TURN_LEFT:
      printf("Turn left %ld\r\n", mvmt->ticks);
      Init_Drive_Turn(mvmt->ticks);
      break;
    case TURN_RIGHT:
      printf("Turn right %ld\r\n", mvmt->ticks);
      Init_Drive_Turn(-(mvmt->ticks));
      break;
  }
}

static unsigned char Defensive_Autonomous(Strategy* strategy)
{
  int returnVal = 0, state = 0, stateDone = 0;

  if (strategy->length > 0)
    Defensive_Drive_Init(strategy->steps);
  else
    printf("Completed\r\n");

  while (autonomous_mode)
  {

    if (statusflag.NEW_SPI_DATA)      /* 26.2ms loop area */
    {
      Getdata(&rxdata);

      Camera_Handler();
      Servo_Track();

      if (state < strategy->length)
      {
        if (strategy->steps[state].turn == TURN_NONE)
        {
          stateDone = Drive_Ticks();
        }
        else if (strategy->steps[state].turn == TURN_LEFT)
        {
          stateDone = Turn_Ticks();
        }
        else
        {
          stateDone = Turn_Ticks();
        }
        
        if (stateDone)
        {
          ++state;
          stateDone = 0;

          if (state < strategy->length)
            Defensive_Drive_Init(strategy->steps + state);
          else
            printf("Completed\r\n");
        }
      }
      else 
      {
        pwm_right_drive = pwm_left_drive = 127;
      }
      
      Putdata(&txdata);
    }

    Process_Data_From_Local_IO();
  }
  
  return returnVal;
}
